@@ -32,15 +32,12 @@
 #  include <config.h>
 #endif
 
-#include <sys/param.h>
-
 #include <glib.h>
 #include <glib/gstdio.h>
 #include <glib/gi18n-lib.h>
 
 #include "brasero-track-data.h"
 #include "burn-mkisofs-base.h"
-#include "burn-debug.h"
 
 typedef struct _BraseroTrackDataPrivate BraseroTrackDataPrivate;
 struct _BraseroTrackDataPrivate
@@ -129,9 +126,9 @@ brasero_track_data_set_source_real (BraseroTrackData *track,
 
 /**
  * brasero_track_data_set_source:
- * @track: a #BraseroTrackData
- * @grafts: a #GSList of #BraseroGraftPt
- * @unreadable: a #GSList of URIS as strings
+ * @track: a #BraseroTrackData.
+ * @grafts: (element-type BraseroBurn.GraftPt) (in) (transfer full): a #GSList of #BraseroGraftPt.
+ * @unreadable: (element-type utf8) (allow-none) (in) (transfer full): a #GSList of URIS as strings or %NULL.
  *
  * Sets the lists of grafts points (@grafts) and excluded
  * URIs (@unreadable) to be used to create an image.
@@ -337,161 +334,26 @@ brasero_track_data_get_fs_real (BraseroTrackData *track)
 	return priv->fs_type;
 }
 
-static GHashTable *
-brasero_track_data_mangle_joliet_name (GHashTable *joliet,
-				       const gchar *path,
-				       gchar *buffer)
-{
-	gboolean has_slash = FALSE;
-	gint dot_pos = -1;
-	gint dot_len = -1;
-	gchar *name;
-	gint width;
-	gint start;
-	gint num;
-	gint end;
-
-	/* NOTE: this wouldn't work on windows (not a big deal) */
-	end = strlen (path);
-	if (!end) {
-		buffer [0] = '\0';
-		return joliet;
-	}
-
-	memcpy (buffer, path, MIN (end, MAXPATHLEN));
-	buffer [MIN (end, MAXPATHLEN)] = '\0';
-
-	/* move back until we find a character different from G_DIR_SEPARATOR */
-	end --;
-	while (end >= 0 && G_IS_DIR_SEPARATOR (path [end])) {
-		end --;
-		has_slash = TRUE;
-	}
-
-	/* There are only slashes */
-	if (end == -1)
-		return joliet;
-
-	start = end - 1;
-	while (start >= 0 && !G_IS_DIR_SEPARATOR (path [start])) {
-		/* Find the extension while at it */
-		if (dot_pos <= 0 && path [start] == '.')
-			dot_pos = start;
-
-		start --;
-	}
-
-	if (end - start <= 64)
-		return joliet;
-
-	name = buffer + start + 1;
-	if (dot_pos > 0)
-		dot_len = end - dot_pos + 1;
-
-	if (dot_len > 1 && dot_len < 5)
-		memcpy (name + 64 - dot_len,
-			path + dot_pos,
-			dot_len);
-
-	name [64] = '\0';
-
-	if (!joliet) {
-		joliet = g_hash_table_new_full (g_str_hash,
-						g_str_equal,
-						g_free,
-						NULL);
-
-		g_hash_table_insert (joliet, g_strdup (buffer), GINT_TO_POINTER (1));
-		if (has_slash)
-			strcat (buffer, G_DIR_SEPARATOR_S);
-
-		BRASERO_BURN_LOG ("Mangled name to %s (truncated)", buffer);
-		return joliet;
-	}
-
-	/* see if this path was already used */
-	num = GPOINTER_TO_INT (g_hash_table_lookup (joliet, buffer));
-	if (!num) {
-		g_hash_table_insert (joliet, g_strdup (buffer), GINT_TO_POINTER (1));
-
-		if (has_slash)
-			strcat (buffer, G_DIR_SEPARATOR_S);
-
-		BRASERO_BURN_LOG ("Mangled name to %s (truncated)", buffer);
-		return joliet;
-	}
-
-	/* NOTE: g_hash_table_insert frees key_path */
-	num ++;
-	g_hash_table_insert (joliet, g_strdup (buffer), GINT_TO_POINTER (num));
-
-	width = 1;
-	while (num / (width * 10)) width ++;
-
-	/* try to keep the extension */
-	if (dot_len < 5 && dot_len > 1 )
-		sprintf (name + (64 - width - dot_len),
-			 "%i%s",
-			 num,
-			 path + dot_pos);
-	else
-		sprintf (name + (64 - width),
-			 "%i",
-			 num);
-
-	if (has_slash)
-		strcat (buffer, G_DIR_SEPARATOR_S);
-
-	BRASERO_BURN_LOG ("Mangled name to %s", buffer);
-	return joliet;
-}
-
 /**
- * brasero_track_data_get_fs:
+ * brasero_track_data_get_grafts:
  * @track: a #BraseroTrackData
  *
  * Returns a list of #BraseroGraftPt.
  *
  * Do not free after usage as @track retains ownership.
  *
- * Return value: a #GSList of #BraseroGraftPt or NULL if empty.
+ * Return value: (transfer none) (element-type BraseroBurn.GraftPt) (allow-none): a #GSList of #BraseroGraftPt or %NULL if empty.
  **/
 
 GSList *
 brasero_track_data_get_grafts (BraseroTrackData *track)
 {
 	BraseroTrackDataClass *klass;
-	GHashTable *mangle = NULL;
-	BraseroImageFS image_fs;
-	GSList *grafts;
-	GSList *iter;
 
 	g_return_val_if_fail (BRASERO_IS_TRACK_DATA (track), NULL);
 
 	klass = BRASERO_TRACK_DATA_GET_CLASS (track);
-	grafts = klass->get_grafts (track);
-
-	image_fs = brasero_track_data_get_fs (track);
-	if ((image_fs & BRASERO_IMAGE_FS_JOLIET) == 0)
-		return grafts;
-
-	for (iter = grafts; iter; iter = iter->next) {
-		BraseroGraftPt *graft;
-		gchar newpath [MAXPATHLEN];
-
-		graft = grafts->data;
-		mangle = brasero_track_data_mangle_joliet_name (mangle,
-								graft->path,
-								newpath);
-
-		g_free (graft->path);
-		graft->path = g_strdup (newpath);
-	}
-
-	if (mangle)
-		g_hash_table_destroy (mangle);
-
-	return grafts;
+	return klass->get_grafts (track);
 }
 
 static GSList *
@@ -504,21 +366,47 @@ brasero_track_data_get_grafts_real (BraseroTrackData *track)
 }
 
 /**
- * brasero_track_data_get_excluded:
- * @track: a #BraseroTrackData
- * @copy: a #gboolean
+ * brasero_track_data_get_excluded_list:
+ * @track: a #BraseroTrackData.
  *
  * Returns a list of URIs which must not be included in
  * the image to be created.
- * If @copy is TRUE then the @list is a copy and must
- * be freed once it is not needed anymore. If FALSE,
- * do not free after usage as @track retains ownership.
+ * Do not free the list or any of the URIs after
+ * usage as @track retains ownership.
  *
- * Return value: a #GSList of #gchar * or NULL if no
+ * Return value: (transfer none) (element-type utf8) (allow-none): a #GSList of #gchar * or %NULL if no
  * URI should be excluded.
  **/
 
 GSList *
+brasero_track_data_get_excluded_list (BraseroTrackData *track)
+{
+	BraseroTrackDataClass *klass;
+
+	g_return_val_if_fail (BRASERO_IS_TRACK_DATA (track), NULL);
+
+	klass = BRASERO_TRACK_DATA_GET_CLASS (track);
+	return klass->get_excluded (track);
+}
+
+/**
+ * brasero_track_data_get_excluded:
+ * @track: a #BraseroTrackData.
+ * @copy: a #gboolean.
+ *
+ * Returns a list of URIs which must not be included in
+ * the image to be created.
+ * If @copy is %TRUE then the @list is a copy and must
+ * be freed once it is not needed anymore. If %FALSE,
+ * do not free after usage as @track retains ownership.
+ *
+ * Deprecated since 2.29.2
+ *
+ * Return value: a #GSList of #gchar * or %NULL if no
+ * URI should be excluded.
+ **/
+
+G_GNUC_DEPRECATED GSList *
 brasero_track_data_get_excluded (BraseroTrackData *track,
 				 gboolean copy)
 {
@@ -555,13 +443,13 @@ brasero_track_data_get_excluded_real (BraseroTrackData *track)
 
 /**
  * brasero_track_data_get_paths:
- * @track: a #BraseroTrackData
- * @use_joliet: a #gboolean
- * @grafts_path: a #gchar
- * @excluded_path: a #gchar
- * @emptydir: a #gchar
- * @videodir: a #gchar or NULL
- * @error: a #GError
+ * @track: a #BraseroTrackData.
+ * @use_joliet: a #gboolean.
+ * @grafts_path: a #gchar.
+ * @excluded_path: a #gchar.
+ * @emptydir: a #gchar.
+ * @videodir: (allow-none): a #gchar or %NULL.
+ * @error: a #GError.
  *
  * Write in @grafts_path (a path to a file) the graft points,
  * in @excluded_path (a path to a file) the list of paths to
@@ -574,10 +462,12 @@ brasero_track_data_get_excluded_real (BraseroTrackData *track)
  *
  * This function takes care of mangling.
  *
+ * Deprecated since 2.29.2
+ *
  * Return value: a #BraseroBurnResult.
  **/
 
-BraseroBurnResult
+G_GNUC_DEPRECATED BraseroBurnResult
 brasero_track_data_get_paths (BraseroTrackData *track,
 			      gboolean use_joliet,
 			      const gchar *grafts_path,
@@ -608,15 +498,68 @@ brasero_track_data_get_paths (BraseroTrackData *track,
 	return result;
 }
 
+/**
+ * brasero_track_data_write_to_paths:
+ * @track: a #BraseroTrackData.
+ * @grafts_path: a #gchar.
+ * @excluded_path: a #gchar.
+ * @emptydir: a #gchar.
+ * @videodir: (allow-none): a #gchar or %NULL.
+ * @error: a #GError.
+ *
+ * Write to @grafts_path (a path to a file) the graft points,
+ * and to @excluded_path (a path to a file) the list of paths to
+ * be excluded; @emptydir is (path) is an empty
+ * directory to be used for created directories;
+ * @videodir (a path) is a directory to be used to build the
+ * the video image.
+ *
+ * This is mostly for internal use by mkisofs and similar.
+ *
+ * This function takes care of file name mangling.
+ *
+ * Return value: a #BraseroBurnResult.
+ **/
+
+BraseroBurnResult
+brasero_track_data_write_to_paths (BraseroTrackData *track,
+                                   const gchar *grafts_path,
+                                   const gchar *excluded_path,
+                                   const gchar *emptydir,
+                                   const gchar *videodir,
+                                   GError **error)
+{
+	GSList *grafts;
+	GSList *excluded;
+	BraseroBurnResult result;
+	BraseroTrackDataClass *klass;
+
+	g_return_val_if_fail (BRASERO_IS_TRACK_DATA (track), BRASERO_BURN_NOT_SUPPORTED);
+
+	klass = BRASERO_TRACK_DATA_GET_CLASS (track);
+	grafts = klass->get_grafts (track);
+	excluded = klass->get_excluded (track);
+
+	result = brasero_mkisofs_base_write_to_files (grafts,
+						      excluded,
+						      brasero_track_data_get_fs (track),
+						      emptydir,
+						      videodir,
+						      grafts_path,
+						      excluded_path,
+						      error);
+	return result;
+}
+
 /**
  * brasero_track_data_get_file_num:
- * @track: a #BraseroTrackData
- * @file_num: a #guint64
+ * @track: a #BraseroTrackData.
+ * @file_num: (allow-none) (out): a #guint64 or %NULL.
  *
  * Sets the number of files (not directories) in @file_num.
  *
- * Return value: a #BraseroBurnResult. TRUE if @file_num
- * was set, FALSE otherwise.
+ * Return value: a #BraseroBurnResult. %TRUE if @file_num
+ * was set, %FALSE otherwise.
  **/
 
 BraseroBurnResult
@@ -661,7 +604,7 @@ brasero_track_data_get_size (BraseroTrack *track,
 	return BRASERO_BURN_OK;
 }
 
-static BraseroTrackDataType
+static BraseroBurnResult
 brasero_track_data_get_track_type (BraseroTrack *track,
 				   BraseroTrackType *type)
 {
@@ -669,13 +612,10 @@ brasero_track_data_get_track_type (BraseroTrack *track,
 
 	priv = BRASERO_TRACK_DATA_PRIVATE (track);
 
-	if (!type)
-		return BRASERO_TRACK_TYPE_DATA;
-
 	brasero_track_type_set_has_data (type);
 	brasero_track_type_set_data_fs (type, priv->fs_type);
 
-	return BRASERO_TRACK_TYPE_DATA;
+	return BRASERO_BURN_OK;
 }
 
 static BraseroBurnResult
