@@ -1,20 +1,23 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /*
  * Brasero
- * Copyright (C) Philippe Rouquier 2008 <bonfire-app@wanadoo.fr>
+ * Copyright (C) Philippe Rouquier 2005-2009 <bonfire-app@wanadoo.fr>
  * 
- * Brasero is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ *  Brasero is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  * 
- * Brasero is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * brasero is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License along
- * with this program. If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU General Public License
+ * along with brasero.  If not, write to:
+ * 	The Free Software Foundation, Inc.,
+ * 	51 Franklin Street, Fifth Floor
+ * 	Boston, MA  02110-1301, USA.
  */
 
 #ifndef _BRASERO_APP_H_
@@ -23,8 +26,7 @@
 #include <glib-object.h>
 #include <gtk/gtk.h>
 
-#include "burn-session.h"
-
+#include "brasero-session-cfg.h"
 
 G_BEGIN_DECLS
 
@@ -66,7 +68,7 @@ brasero_app_set_toplevel (BraseroApp *app, GtkWindow *window);
 void
 brasero_app_create_mainwin (BraseroApp *app);
 
-void
+gboolean
 brasero_app_run_mainwin (BraseroApp *app);
 
 gboolean
@@ -88,22 +90,63 @@ gboolean
 brasero_app_burn (BraseroApp *app,
 		  BraseroBurnSession *session);
 
+gboolean
+brasero_app_burn_options (BraseroApp *app,
+			  BraseroSessionCfg *session);
+
 void
-brasero_app_burn_image (BraseroApp *app,
-			const gchar *uri);
+brasero_app_burn_uri (BraseroApp *app,
+		      BraseroDrive *burner,
+		      gboolean burn);
+
+void
+brasero_app_data (BraseroApp *app,
+		  BraseroDrive *burner,
+		  gchar * const *uris,
+		  gboolean burn);
+
+void
+brasero_app_stream (BraseroApp *app,
+		    BraseroDrive *burner,
+		    gchar * const *uris,
+		    gboolean is_video,
+		    gboolean burn);
+
+void
+brasero_app_image (BraseroApp *app,
+		   BraseroDrive *burner,
+		   const gchar *uri,
+		   gboolean burn);
 
 void
 brasero_app_copy_disc (BraseroApp *app,
+		       BraseroDrive *burner,
 		       const gchar *device,
-		       const gchar *cover);
+		       const gchar *cover,
+		       gboolean burn);
 
 void
 brasero_app_blank (BraseroApp *app,
-		   const gchar *device);
+		   BraseroDrive *burner,
+		   gboolean burn);
 
 void
 brasero_app_check (BraseroApp *app,
-		   const gchar *device);
+		   BraseroDrive *burner,
+		   gboolean burn);
+
+gboolean
+brasero_app_open_project (BraseroApp *app,
+			  BraseroDrive *burner,
+                          const gchar *uri,
+                          gboolean is_playlist,
+                          gboolean warn_user,
+                          gboolean burn);
+
+gboolean
+brasero_app_open_uri (BraseroApp *app,
+                      const gchar *uri_arg,
+                      gboolean warn_user);
 
 GtkWidget *
 brasero_app_get_statusbar1 (BraseroApp *app);
