@@ -1,3 +1,5 @@
+/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
+
 /***************************************************************************
  *            project.c
  *
@@ -37,6 +39,7 @@
 #include <gio/gio.h>
 
 #include <gtk/gtk.h>
+#include <gdk/gdkx.h>
 
 #include <gst/gst.h>
 
@@ -44,6 +47,7 @@
 
 #include "brasero-misc.h"
 #include "brasero-jacket-edit.h"
+#include "brasero-pk.h"
 
 #include "brasero-tags.h"
 #include "brasero-session.h"
@@ -67,6 +71,7 @@
 #include "brasero-video-options.h"
 #include "brasero-drive-properties.h"
 #include "brasero-image-properties.h"
+#include "burn-plugin-manager.h"
 
 #include "brasero-project-type-chooser.h"
 #include "brasero-app.h"
@@ -164,6 +169,8 @@ struct BraseroProjectPrivate {
 
 	BraseroURIContainer *current_source;
 
+	GCancellable *cancel;
+
 	GtkWidget *chooser;
 	gulong selected_id;
 	gulong activated_id;
@@ -328,10 +335,14 @@ brasero_project_get_proportion (BraseroLayoutObject *object,
 				gint *center,
 				gint *footer)
 {
+	GtkAllocation allocation;
+
 	if (!BRASERO_PROJECT (object)->priv->name_display)
 		return;
 
-	*footer = BRASERO_PROJECT (object)->priv->name_display->parent->allocation.height;
+	gtk_widget_get_allocation (gtk_widget_get_parent (BRASERO_PROJECT (object)->priv->name_display),
+				   &allocation);
+	*footer = allocation.height;
 }
 
 static void
@@ -375,7 +386,8 @@ brasero_project_set_add_button_state (BraseroProject *project)
 		return;
 
 	widget = gtk_bin_get_child (GTK_BIN (widget));
-	GTK_WIDGET_SET_FLAGS (widget, GTK_CAN_DEFAULT);
+	gtk_widget_set_can_default (widget, TRUE);
+
 	gtk_window_set_default (GTK_WINDOW (toplevel), widget);
 }
 
@@ -573,7 +585,7 @@ brasero_utils_disc_style_changed_cb (GtkWidget *widget,
 {
 	/* The widget (a treeview here) needs to be realized to get proper style */
 	gtk_widget_realize (widget);
-	gtk_widget_modify_bg (event_box, GTK_STATE_NORMAL, &widget->style->base[GTK_STATE_NORMAL]);
+	gtk_widget_modify_bg (event_box, GTK_STATE_NORMAL, &gtk_widget_get_style (widget)->base[GTK_STATE_NORMAL]);
 }
 
 static void
@@ -582,7 +594,7 @@ brasero_utils_disc_realized_cb (GtkWidget *event_box,
 {
 	/* The widget (a treeview here) needs to be realized to get proper style */
 	gtk_widget_realize (textview);
-	gtk_widget_modify_bg (event_box, GTK_STATE_NORMAL, &textview->style->base[GTK_STATE_NORMAL]);
+	gtk_widget_modify_bg (event_box, GTK_STATE_NORMAL, &gtk_widget_get_style (textview)->base[GTK_STATE_NORMAL]);
 
 	g_signal_handlers_disconnect_by_func (textview,
 					      brasero_utils_disc_style_changed_cb,
@@ -652,7 +664,7 @@ brasero_disc_get_use_info_notebook (BraseroProject *project)
 	gtk_container_add (GTK_CONTAINER (frame), event_box);
 
 	/* The alignment to set properly the position of the GtkTextView */
-	alignment = gtk_alignment_new (0.5, 0.3, 0.0, 0.0);
+	alignment = gtk_alignment_new (0.5, 0.5, 1.0, 0.0);
 	gtk_container_set_border_width (GTK_CONTAINER (alignment), 10);
 	gtk_widget_show (alignment);
 	gtk_container_add (GTK_CONTAINER (event_box), alignment);
@@ -660,49 +672,16 @@ brasero_disc_get_use_info_notebook (BraseroProject *project)
 	/* The TreeView for the message */
 	buffer = gtk_text_buffer_new (NULL);
 	gtk_text_buffer_create_tag (buffer, "Title",
-	                            "scale", 1.2,
-	                            "justification", GTK_JUSTIFY_LEFT,
-	                            "foreground", "grey50",
-	                            NULL);
-
-	gtk_text_buffer_create_tag (buffer, "TextBody",
-	                            "justification", GTK_JUSTIFY_LEFT,
+	                            "scale", 1.1,
+	                            "justification", GTK_JUSTIFY_CENTER,
 	                            "foreground", "grey50",
+	                            "wrap-mode", GTK_WRAP_WORD,
 	                            NULL);
 
 	gtk_text_buffer_get_start_iter (buffer, &iter);
-
-	/* Translators: this messages will appear as a list of possible
-	 * actions, like:
-	 *   To add/remove files you can:
-         *      * perform action one
-         *      * perform action two
-	 * The full message will be showed in the main area of an empty
-	 * project, suggesting users how to add and remove items to project.
-	 * You simply have to translate messages in the best form
-         * for a list of actions. */
-	gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, _("To add files to this project you can:"), -1, "Title", NULL);
-	gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, "\n\t* ", -1, "TextBody", NULL);
-
-	gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, _("click the \"Add\" button to show a selection dialog"), -1, "TextBody", NULL);
-	gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, "\n\t* ", -1, "TextBody", NULL);
-	gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, _("select files in the selection pane and click the \"Add\" button"), -1, "TextBody", NULL);
-	gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, "\n\t* ", -1, "TextBody", NULL);
-	gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, _("drag files in this area from the selection pane or from the file manager"), -1, "TextBody", NULL);
-	gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, "\n\t* ", -1, "TextBody", NULL);
-	gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, _("double-click on files in the selection pane"), -1, "TextBody", NULL);
-	gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, "\n\t* ", -1, "TextBody", NULL);
-	gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, _("copy files (from file manager for example) and paste in this area"), -1, "TextBody", NULL);
+	gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, _("To add files to this project click the \"Add\" button or drag files to this area"), -1, "Title", NULL);
 	gtk_text_buffer_insert (buffer, &iter, "\n\n\n", -1);
-
-	gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, _("To remove files from this project you can:"), -1, "Title", NULL);
-	gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, "\n\t* ", -1, "TextBody", NULL);
-
-	gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, _("click on the \"Remove\" button to remove selected items in this area"), -1, "TextBody", NULL);
-	gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, "\n\t* ", -1, "TextBody", NULL);
-	gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, _("select items in this area, and choose \"Remove\" from context menu"), -1, "TextBody", NULL);
-	gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, "\n\t* ", -1, "TextBody", NULL);
-	gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, _("select items in this area, and press \"Delete\" key"), -1, "TextBody", NULL);
+	gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, _("To remove files select them then click on the \"Remove\" button or press \"Delete\" key"), -1, "Title", NULL);
 
 	textview = gtk_text_view_new_with_buffer (buffer);
 	gtk_text_view_set_editable (GTK_TEXT_VIEW (textview), FALSE);
@@ -890,27 +869,28 @@ brasero_project_is_valid (BraseroSessionCfg *session,
 			  BraseroProject *project)
 {
 	BraseroSessionError valid;
-	BraseroStatus *status;
 	GdkWindow *window;
 	GdkCursor *cursor;
 	GtkAction *action;
 
 	/* Update the cursor */
-	status = brasero_status_new ();
-	brasero_burn_session_get_status (BRASERO_BURN_SESSION (session), status);
-
 	window = gtk_widget_get_window (GTK_WIDGET (project));
 	if (window) {
-		if (brasero_status_get_result (status) == BRASERO_BURN_NOT_READY) {
+		BraseroStatus *status;
+
+		status = brasero_status_new ();
+		brasero_burn_session_get_status (BRASERO_BURN_SESSION (session), status);
+		if (brasero_status_get_result (status) == BRASERO_BURN_NOT_READY
+		||  brasero_status_get_result (status) == BRASERO_BURN_RUNNING) {
 			cursor = gdk_cursor_new (GDK_WATCH);
 			gdk_window_set_cursor (window, cursor);
 			gdk_cursor_unref (cursor);
 		}
 		else
 			gdk_window_set_cursor (window, NULL);
-	}
 
-	brasero_status_free (status);
+		g_object_unref (status);
+	}
 
 	valid = brasero_session_cfg_get_error (project->priv->session);
 
@@ -925,7 +905,7 @@ brasero_project_is_valid (BraseroSessionCfg *session,
 	 * that means that file selection is not empty */
 
 	/* Clean any message */
-	brasero_notify_message_remove (BRASERO_NOTIFY (project->priv->message),
+	brasero_notify_message_remove (project->priv->message,
 				       BRASERO_NOTIFY_CONTEXT_SIZE);
 
 	if (valid == BRASERO_SESSION_EMPTY) {
@@ -942,7 +922,7 @@ brasero_project_is_valid (BraseroSessionCfg *session,
 		min_disc_size = brasero_session_span_get_max_space (BRASERO_SESSION_SPAN (session));
 
 		/* One rule should be that the maximum batch size should not exceed the disc size
-		 * FIXME: we could change it into a dialog telling the user what is the maximum
+		 * FIXME! we could change it into a dialog telling the user what is the maximum
 		 * size required. */
 		available_space = brasero_burn_session_get_available_medium_space (BRASERO_BURN_SESSION (session));
 
@@ -952,16 +932,15 @@ brasero_project_is_valid (BraseroSessionCfg *session,
 		&& brasero_session_span_possible (BRASERO_SESSION_SPAN (session)) == BRASERO_BURN_RETRY) {
 			GtkWidget *message;
 
-			message = brasero_notify_message_add (BRASERO_NOTIFY (project->priv->message),
+			message = brasero_notify_message_add (project->priv->message,
 							      _("Would you like to burn the selection of files across several media?"),
 							      _("The project is too large for the disc even with the overburn option."),
 							      -1,
 							      BRASERO_NOTIFY_CONTEXT_SIZE);
-			brasero_notify_button_add (BRASERO_NOTIFY (project->priv->message),
-						   BRASERO_DISC_MESSAGE (message),
-						   _("_Burn Several Discs"),
-						   _("Burn the selection of files across several media"),
-						   GTK_RESPONSE_OK);
+			gtk_widget_set_tooltip_text (gtk_info_bar_add_button (GTK_INFO_BAR (message),
+									      _("_Burn Several Discs"),
+								    	      GTK_RESPONSE_OK),
+						     _("Burn the selection of files across several media"));
 
 			g_signal_connect (message,
 					  "response",
@@ -969,7 +948,7 @@ brasero_project_is_valid (BraseroSessionCfg *session,
 					  project);
 		}
 		else
-			brasero_notify_message_add (BRASERO_NOTIFY (project->priv->message),
+			brasero_notify_message_add (project->priv->message,
 						    _("Please choose another CD or DVD or insert a new one."),
 						    _("The project is too large for the disc even with the overburn option."),
 						    -1,
@@ -980,18 +959,17 @@ brasero_project_is_valid (BraseroSessionCfg *session,
 
 		project->priv->empty = FALSE;
 		project->priv->oversized = TRUE;
-		message = brasero_notify_message_add (BRASERO_NOTIFY (project->priv->message),
+		message = brasero_notify_message_add (project->priv->message,
 						      _("Would you like to burn beyond the disc's reported capacity?"),
 						      _("The project is too large for the disc and you must remove files from it."
 							"\nYou may want to use this option if you're using 90 or 100 min CD-R(W) which cannot be properly recognized and therefore needs the overburn option."
 							"\nNote: This option might cause failure."),
 						      -1,
 						      BRASERO_NOTIFY_CONTEXT_SIZE);
-		brasero_notify_button_add (BRASERO_NOTIFY (project->priv->message),
-					   BRASERO_DISC_MESSAGE (message),
-					   _("_Overburn"),
-					   _("Burn beyond the disc's reported capacity"),
-					   GTK_RESPONSE_OK);
+		gtk_widget_set_tooltip_text (gtk_info_bar_add_button (GTK_INFO_BAR (message),
+								      _("_Overburn"),
+							    	      GTK_RESPONSE_OK),
+					     _("Burn beyond the disc's reported capacity"));
 
 		g_signal_connect (message,
 				  "response",
@@ -1000,7 +978,7 @@ brasero_project_is_valid (BraseroSessionCfg *session,
 	}
 	else if (valid == BRASERO_SESSION_NO_OUTPUT) {
 		project->priv->empty = FALSE;
-		brasero_notify_message_add (BRASERO_NOTIFY (project->priv->message),
+		brasero_notify_message_add (project->priv->message,
 					    _("Please insert a writable CD or DVD."),
 					    NULL,
 					    -1,
@@ -1008,14 +986,14 @@ brasero_project_is_valid (BraseroSessionCfg *session,
 	}
 	else if (valid == BRASERO_SESSION_NOT_SUPPORTED) {
 		project->priv->empty = FALSE;
-		brasero_notify_message_add (BRASERO_NOTIFY (project->priv->message),
+		brasero_notify_message_add (project->priv->message,
 					    _("Please replace the disc with a supported CD or DVD."),
-					    _("It is not possible to write with the current set of plugins."),
+					    NULL,
 					    -1,
 					    BRASERO_NOTIFY_CONTEXT_SIZE);
 	}
 	else if (valid == BRASERO_SESSION_NO_CD_TEXT) {
-		brasero_notify_message_add (BRASERO_NOTIFY (project->priv->message),
+		brasero_notify_message_add (project->priv->message,
 					    _("No track information (artist, title, ...) will be written to the disc."),
 					    _("This is not supported by the current active burning backend."),
 					    -1,
@@ -1025,7 +1003,7 @@ brasero_project_is_valid (BraseroSessionCfg *session,
 	     &&  brasero_medium_selection_get_media_num (BRASERO_MEDIUM_SELECTION (project->priv->selection)) == 1) {
 		/* The user may have forgotten to insert a disc so remind him of that if
 		 * there aren't any other possibility in the selection */
-		brasero_notify_message_add (BRASERO_NOTIFY (project->priv->message),
+		brasero_notify_message_add (project->priv->message,
 					    _("Please insert a writable CD or DVD if you don't want to write to an image file."),
 					    NULL,
 					    10000,
@@ -1190,6 +1168,11 @@ brasero_project_finalize (GObject *object)
 	BraseroProject *cobj;
 	cobj = BRASERO_PROJECT(object);
 
+	if (cobj->priv->cancel) {
+		g_cancellable_cancel (cobj->priv->cancel);
+		cobj->priv->cancel = NULL;
+	}
+
 	if (cobj->priv->session) {
 		g_object_unref (cobj->priv->session);
 		cobj->priv->session = NULL;
@@ -1293,20 +1276,20 @@ brasero_project_check_status (BraseroProject *project)
         status = brasero_status_new ();
         brasero_burn_session_get_status (BRASERO_BURN_SESSION (project->priv->session), status);
         result = brasero_status_get_result (status);
-        brasero_status_free (status);
+        g_object_unref (status);
 
         if (result == BRASERO_BURN_ERR) {
                 /* At the moment the only error possible is an empty project */
-                if (BRASERO_IS_AUDIO_DISC (project->priv->current))
-                        brasero_project_no_song_dialog (project);
-                else
-                        brasero_project_no_file_dialog (project);
+		if (BRASERO_IS_AUDIO_DISC (project->priv->current))
+			brasero_project_no_song_dialog (project);
+		else
+			brasero_project_no_file_dialog (project);
 
-                return BRASERO_BURN_ERR;
-        }
+		return BRASERO_BURN_ERR;
+	}
 
-        if (result == BRASERO_BURN_OK)
-                return BRASERO_BURN_OK;
+	if (result == BRASERO_BURN_OK)
+		return BRASERO_BURN_OK;
 
         dialog = brasero_status_dialog_new (BRASERO_BURN_SESSION (project->priv->session),
                                                                   gtk_widget_get_toplevel (GTK_WIDGET (project)));
@@ -1318,6 +1301,124 @@ brasero_project_check_status (BraseroProject *project)
         return (response == GTK_RESPONSE_OK)? BRASERO_BURN_OK:BRASERO_BURN_CANCEL;
 }
 
+static BraseroBurnResult
+brasero_project_install_missing (BraseroPluginErrorType type,
+                                 const gchar *detail,
+                                 gpointer user_data)
+{
+	BraseroProject *project = BRASERO_PROJECT (user_data);
+	GCancellable *cancel;
+	BraseroPK *package;
+	GtkWidget *parent;
+	gboolean res;
+	int xid = 0;
+
+	/* Get the xid */
+	parent = gtk_widget_get_toplevel (GTK_WIDGET (project));
+	xid = gdk_x11_drawable_get_xid (GDK_DRAWABLE (GTK_WIDGET (parent)->window));
+
+	package = brasero_pk_new ();
+	cancel = g_cancellable_new ();
+	project->priv->cancel = cancel;
+	switch (type) {
+		case BRASERO_PLUGIN_ERROR_MISSING_APP:
+			res = brasero_pk_install_missing_app (package, detail, xid, cancel);
+			break;
+
+		case BRASERO_PLUGIN_ERROR_MISSING_LIBRARY:
+			res = brasero_pk_install_missing_library (package, detail, xid, cancel);
+			break;
+
+		case BRASERO_PLUGIN_ERROR_MISSING_GSTREAMER_PLUGIN:
+			res = brasero_pk_install_gstreamer_plugin (package, detail, xid, cancel);
+			break;
+
+		default:
+			res = FALSE;
+			break;
+	}
+
+	if (package) {
+		g_object_unref (package);
+		package = NULL;
+	}
+
+	if (g_cancellable_is_cancelled (cancel)) {
+		g_object_unref (cancel);
+		return BRASERO_BURN_CANCEL;
+	}
+
+	project->priv->cancel = NULL;
+	g_object_unref (cancel);
+
+	if (!res)
+		return BRASERO_BURN_ERR;
+
+	return BRASERO_BURN_RETRY;
+}
+
+static BraseroBurnResult
+brasero_project_list_missing (BraseroPluginErrorType type,
+                              const gchar *detail,
+                              gpointer user_data)
+{
+	GString *string = user_data;
+
+	if (type == BRASERO_PLUGIN_ERROR_MISSING_APP) {
+		g_string_append_c (string, '\n');
+		/* Translators: %s is the name of a missing application */
+		g_string_append_printf (string, _("%s (application)"), detail);
+	}
+	else if (type == BRASERO_PLUGIN_ERROR_MISSING_LIBRARY) {
+		g_string_append_c (string, '\n');
+		/* Translators: %s is the name of a missing library */
+		g_string_append_printf (string, _("%s (library)"), detail);
+	}
+	else if (type == BRASERO_PLUGIN_ERROR_MISSING_GSTREAMER_PLUGIN) {
+		g_string_append_c (string, '\n');
+		/* Translators: %s is the name of a missing Gstreamer plugin */
+		g_string_append_printf (string, _("%s (Gstreamer plugin)"), detail);
+	}
+
+	return BRASERO_BURN_OK;
+}
+
+static BraseroBurnResult
+brasero_project_check_plugins_not_ready (BraseroProject *project,
+                                         BraseroBurnSession *session)
+{
+	BraseroBurnResult result;
+	GtkWidget *parent;
+	GString *string;
+
+	parent = gtk_widget_get_toplevel (GTK_WIDGET (project));
+	gtk_widget_set_sensitive (parent, FALSE);
+
+	result = brasero_session_foreach_plugin_error (session,
+	                                               brasero_project_install_missing,
+	                                               project);
+	if (result == BRASERO_BURN_CANCEL)
+		return result;
+
+	gtk_widget_set_sensitive (parent, TRUE);
+
+	if (result == BRASERO_BURN_OK)
+		return result;
+
+	string = g_string_new (_("Please install the following manually and try again:"));
+	brasero_session_foreach_plugin_error (session,
+	                                      brasero_project_list_missing,
+	                                      string);
+
+	brasero_utils_message_dialog (parent,
+	                              _("All required applications and libraries are not installed."),
+	                              string->str,
+	                              GTK_MESSAGE_ERROR);
+	g_string_free (string, TRUE);
+
+	return BRASERO_BURN_ERR;
+}
+
 /******************************** burning **************************************/
 static void
 brasero_project_setup_session (BraseroProject *project,
@@ -1357,6 +1458,7 @@ brasero_project_drive_properties (BraseroProject *project)
 
 	/* Build dialog */
 	drive = brasero_burn_session_get_burner (BRASERO_BURN_SESSION (project->priv->session));
+
 	display_name = brasero_drive_get_display_name (drive);
 	header = g_strdup_printf (_("Properties of %s"), display_name);
 	g_free (display_name);
@@ -1439,9 +1541,7 @@ brasero_project_image_properties (BraseroProject *project)
 				      button,
 				      GTK_RESPONSE_OK);
 
-	GTK_WIDGET_SET_FLAGS (button,
-	                      GTK_WIDGET_FLAGS (button)|
-	                      GTK_CAN_DEFAULT);
+	gtk_widget_set_can_default (button, TRUE);
 
 	brasero_image_properties_set_session (BRASERO_IMAGE_PROPERTIES (dialog), project->priv->session);
 
@@ -1478,6 +1578,14 @@ brasero_project_burn (BraseroProject *project)
 	gboolean res = FALSE;
 	BraseroDisc *current_disc;
 
+	/* Check that we are ready */
+	if (brasero_project_check_status (project) != BRASERO_BURN_OK)
+		return;
+
+	/* Check that we are not missing any plugin */
+	if (brasero_project_check_plugins_not_ready (project, BRASERO_BURN_SESSION (project->priv->session)) != BRASERO_BURN_OK)
+		return;
+
 	if (!brasero_burn_session_is_dest_file (BRASERO_BURN_SESSION (project->priv->session)))
 		res = brasero_project_drive_properties (project);
 	else
@@ -1592,9 +1700,9 @@ brasero_project_reset (BraseroProject *project)
 		project->priv->session = NULL;
 	}
 
-	brasero_notify_message_remove (BRASERO_NOTIFY (project->priv->message), BRASERO_NOTIFY_CONTEXT_SIZE);
-	brasero_notify_message_remove (BRASERO_NOTIFY (project->priv->message), BRASERO_NOTIFY_CONTEXT_LOADING);
-	brasero_notify_message_remove (BRASERO_NOTIFY (project->priv->message), BRASERO_NOTIFY_CONTEXT_MULTISESSION);
+	brasero_notify_message_remove (project->priv->message, BRASERO_NOTIFY_CONTEXT_SIZE);
+	brasero_notify_message_remove (project->priv->message, BRASERO_NOTIFY_CONTEXT_LOADING);
+	brasero_notify_message_remove (project->priv->message, BRASERO_NOTIFY_CONTEXT_MULTISESSION);
 }
 
 static void
@@ -1609,6 +1717,8 @@ brasero_project_new_session (BraseroProject *project,
 	else
 		project->priv->session = brasero_session_cfg_new ();
 
+	brasero_burn_session_set_strict_support (BRASERO_BURN_SESSION (project->priv->session), FALSE);
+
 	/* NOTE: "is-valid" is emitted whenever there is a change in the
 	 * contents of the session. So no need to connect to track-added, ... */
 	g_signal_connect (project->priv->session,
@@ -1644,6 +1754,7 @@ brasero_project_switch (BraseroProject *project, BraseroProjectType type)
 		gtk_notebook_set_current_page (GTK_NOTEBOOK (project->priv->discs), 0);
 		brasero_medium_selection_show_media_type (BRASERO_MEDIUM_SELECTION (project->priv->selection),
 							  BRASERO_MEDIA_TYPE_WRITABLE|
+							  BRASERO_MEDIA_TYPE_FILE|
 		                                          BRASERO_MEDIA_TYPE_CD);
 	}
 	else if (type == BRASERO_PROJECT_TYPE_DATA) {
@@ -1661,8 +1772,6 @@ brasero_project_switch (BraseroProject *project, BraseroProjectType type)
 							  BRASERO_MEDIA_TYPE_FILE);
 	}
 
-	brasero_dest_selection_choose_best (BRASERO_DEST_SELECTION (project->priv->selection));
-
 	if (project->priv->current) {
 		project->priv->merge_id = brasero_disc_add_ui (project->priv->current,
 							       project->priv->manager,
@@ -1671,7 +1780,7 @@ brasero_project_switch (BraseroProject *project, BraseroProjectType type)
 						   BRASERO_BURN_SESSION (project->priv->session));
 	}
 
-	brasero_notify_message_remove (BRASERO_NOTIFY (project->priv->message), BRASERO_NOTIFY_CONTEXT_SIZE);
+	brasero_notify_message_remove (project->priv->message, BRASERO_NOTIFY_CONTEXT_SIZE);
 
 	/* update the menus */
 	action = gtk_action_group_get_action (project->priv->project_group, "Add");
@@ -1690,46 +1799,24 @@ brasero_project_switch (BraseroProject *project, BraseroProjectType type)
 }
 
 void
-brasero_project_set_audio (BraseroProject *project, GSList *uris)
+brasero_project_set_audio (BraseroProject *project)
 {
 	brasero_project_new_session (project, NULL);
 	brasero_project_switch (project, BRASERO_PROJECT_TYPE_AUDIO);
-
-	for (; uris; uris = uris->next) {
-		gchar *uri;
-
-	    	uri = uris->data;
-		brasero_disc_add_uri (project->priv->current, uri);
-	}
 }
 
 void
-brasero_project_set_data (BraseroProject *project,
-			  GSList *uris)
+brasero_project_set_data (BraseroProject *project)
 {
 	brasero_project_new_session (project, NULL);
 	brasero_project_switch (project, BRASERO_PROJECT_TYPE_DATA);
-
-	for (; uris; uris = uris->next) {
-		gchar *uri;
-
-	    	uri = uris->data;
-		brasero_disc_add_uri (project->priv->current, uri);
-	}
 }
 
 void
-brasero_project_set_video (BraseroProject *project, GSList *uris)
+brasero_project_set_video (BraseroProject *project)
 {
 	brasero_project_new_session (project, NULL);
 	brasero_project_switch (project, BRASERO_PROJECT_TYPE_VIDEO);
-
-	for (; uris; uris = uris->next) {
-		gchar *uri;
-
-	    	uri = uris->data;
-		brasero_disc_add_uri (project->priv->current, uri);
-	}
 }
 
 gboolean
@@ -2199,7 +2286,8 @@ brasero_project_register_ui (BraseroProject *project, GtkUIManager *manager)
 
 	project->priv->manager = manager;
 
-	gtk_widget_grab_focus (project->priv->name_display);
+	if (GTK_WIDGET_REALIZED (project->priv->name_display))
+		gtk_widget_grab_focus (project->priv->name_display);
 }
 
 /******************************* common to save/open ***************************/
