@@ -385,8 +385,12 @@ brasero_video_disc_unreadable_uri_dialog (BraseroVideoDisc *self,
 {
 	gchar *primary;
 	gchar *name;
+	GFile *file;
+
+	file = g_file_new_for_uri (uri);
+	name = g_file_get_basename (file);
+	g_object_unref (file);
 
-	name = g_filename_display_basename (uri);
 	primary = g_strdup_printf (_("\"%s\" could not be opened."), name);
 	brasero_app_alert (brasero_app_get_default (),
 			   primary,
@@ -425,7 +429,7 @@ brasero_video_disc_session_changed (BraseroSessionCfg *session,
 
 	priv = BRASERO_VIDEO_DISC_PRIVATE (self);
 
-	if (!GTK_WIDGET (self)->window)
+	if (!gtk_widget_get_window (GTK_WIDGET (self)))
 		return;
 
 	/* make sure all tracks have video */
@@ -476,7 +480,7 @@ brasero_video_disc_session_changed (BraseroSessionCfg *session,
 			continue;
 		}
 
-		if (result == BRASERO_BURN_NOT_READY) {
+		if (result == BRASERO_BURN_NOT_READY || result == BRASERO_BURN_RUNNING) {
 			notready = TRUE;
 			continue;
 		}
@@ -496,7 +500,7 @@ brasero_video_disc_session_changed (BraseroSessionCfg *session,
 							   BRASERO_TRACK (track));
 		}
 	}
-	brasero_status_free (status);
+	g_object_unref (status);
 }
 
 static void
@@ -1272,7 +1276,8 @@ brasero_video_disc_finalize (GObject *object)
 	priv = BRASERO_VIDEO_DISC_PRIVATE (object);
 
 	if (priv->load_dir) {
-		g_free (priv->load_dir);
+		brasero_io_cancel_by_base (priv->load_dir);
+		brasero_io_job_base_free (priv->load_dir);
 		priv->load_dir = NULL;
 	}	
 
