@@ -154,6 +154,7 @@ brasero_video_tree_model_get_value (GtkTreeModel *model,
 {
 	BraseroVideoTreeModelPrivate *priv;
 	BraseroVideoTreeModel *self;
+	BraseroBurnResult result;
 	BraseroStatus *status;
 	BraseroTrack *track;
 	const gchar *string;
@@ -277,14 +278,15 @@ brasero_video_tree_model_get_value (GtkTreeModel *model,
 		g_value_init (value, G_TYPE_STRING);
 
 		value_tag = NULL;
-		if (brasero_status_get_result (status) == BRASERO_BURN_NOT_READY)
+		result = brasero_status_get_result (status);
+		if (result == BRASERO_BURN_NOT_READY || result == BRASERO_BURN_RUNNING)
 			g_value_set_string (value, "image-loading");
 		else if (brasero_track_tag_lookup (track, BRASERO_TRACK_STREAM_MIME_TAG, &value_tag) == BRASERO_BURN_OK)
 			g_value_set_string (value, g_value_get_string (value_tag));
 		else
 			g_value_set_string (value, "image-missing");
 
-		brasero_status_free (status);
+		g_object_unref (status);
 		return;
 
 	case BRASERO_VIDEO_TREE_MODEL_THUMBNAIL:
@@ -292,8 +294,9 @@ brasero_video_tree_model_get_value (GtkTreeModel *model,
 
 		status = brasero_status_new ();
 		brasero_track_get_status (track, status);
+		result = brasero_status_get_result (status);
 
-		if (brasero_status_get_result (status) == BRASERO_BURN_NOT_READY)
+		if (result == BRASERO_BURN_NOT_READY || result == BRASERO_BURN_RUNNING)
 			pixbuf = gtk_icon_theme_load_icon (priv->theme,
 							   "image-loading",
 							   48,
@@ -318,7 +321,7 @@ brasero_video_tree_model_get_value (GtkTreeModel *model,
 		g_value_set_object (value, pixbuf);
 		g_object_unref (pixbuf);
 
-		brasero_status_free (status);
+		g_object_unref (status);
 		return;
 
 	case BRASERO_VIDEO_TREE_MODEL_SIZE:
@@ -327,7 +330,8 @@ brasero_video_tree_model_get_value (GtkTreeModel *model,
 
 		g_value_init (value, G_TYPE_STRING);
 
-		if (brasero_status_get_result (status) == BRASERO_BURN_OK) {
+		result = brasero_status_get_result (status);
+		if (result == BRASERO_BURN_OK) {
 			guint64 len = 0;
 
 			brasero_track_stream_get_length (BRASERO_TRACK_STREAM (track), &len);
@@ -337,9 +341,9 @@ brasero_video_tree_model_get_value (GtkTreeModel *model,
 			g_free (text);
 		}
 		else
-			g_value_set_string (value, _("(loading ...)"));
+			g_value_set_string (value, _("(loading…)"));
 
-		brasero_status_free (status);
+		g_object_unref (status);
 		return;
 
 	case BRASERO_VIDEO_TREE_MODEL_EDITABLE:
@@ -688,7 +692,7 @@ brasero_video_tree_model_multi_drag_data_get (EggTreeMultiDragSource *drag_sourc
 					      GList *path_list,
 					      GtkSelectionData *selection_data)
 {
-	if (selection_data->target == gdk_atom_intern (BRASERO_DND_TARGET_SELF_FILE_NODES, TRUE)) {
+	if (gtk_selection_data_get_target (selection_data) == gdk_atom_intern (BRASERO_DND_TARGET_SELF_FILE_NODES, TRUE)) {
 		BraseroDNDVideoContext context;
 
 		context.model = GTK_TREE_MODEL (drag_source);
@@ -834,21 +838,23 @@ brasero_video_tree_model_drag_data_received (GtkTreeDragDest *drag_dest,
 {
 	BraseroTrack *sibling;
 	BraseroVideoTreeModelPrivate *priv;
+	GdkAtom target;
 
 	priv = BRASERO_VIDEO_TREE_MODEL_PRIVATE (drag_dest);
 
 	/* The new row(s) must be before dest_path but after our sibling */
 	sibling = brasero_video_tree_model_path_to_track (BRASERO_VIDEO_TREE_MODEL (drag_dest), dest_path);
-		
+
 	/* Received data: see where it comes from:
 	 * - from us, then that's a simple move
 	 * - from another widget then it's going to be URIS and we add
 	 *   them to VideoProject */
-	if (selection_data->target == gdk_atom_intern (BRASERO_DND_TARGET_SELF_FILE_NODES, TRUE)) {
+	target = gtk_selection_data_get_target (selection_data);
+	if (target == gdk_atom_intern (BRASERO_DND_TARGET_SELF_FILE_NODES, TRUE)) {
 		BraseroDNDVideoContext *context;
 		GList *iter;
 
-		context = (BraseroDNDVideoContext *) selection_data->data;
+		context = (BraseroDNDVideoContext *) gtk_selection_data_get_data (selection_data);
 		if (context->model != GTK_TREE_MODEL (drag_dest))
 			return TRUE;
 
@@ -870,18 +876,31 @@ brasero_video_tree_model_drag_data_received (GtkTreeDragDest *drag_dest,
 							      dest_path);
 		}
 	}
-	else if (selection_data->target == gdk_atom_intern ("text/uri-list", TRUE)) {
+	else if (target == gdk_atom_intern ("text/uri-list", TRUE)) {
 		gint i;
-		gchar **uris;
+		gchar **uris = NULL;
 		gboolean success = FALSE;
+		const guchar *selection_data_raw;
+
+		/* NOTE: for some reason gdk_text_property_to_utf8_list_for_display ()
+		 * fails with banshee DND URIs list when calling gtk_selection_data_get_uris ().
+		 * so do like nautilus */
 
 		/* NOTE: there can be many URIs at the same time. One
 		 * success is enough to return TRUE. */
-		success = FALSE;
+		selection_data_raw = gtk_selection_data_get_data (selection_data);
 		uris = gtk_selection_data_get_uris (selection_data);
+		if (!uris) {
+			const guchar *selection_data_raw;
+
+			selection_data_raw = gtk_selection_data_get_data (selection_data);
+			uris = g_uri_list_extract_uris ((gchar *) selection_data_raw);
+		}
+
 		if (!uris)
 			return TRUE;
 
+		success = FALSE;
 		for (i = 0; uris [i]; i ++) {
 			BraseroTrackStreamCfg *track;
 
